<?php

/**
 * Classe TextReplace
 *
 * @version 1.0
 * @date	08/01/2012
 * @author	Humpf
 **/
class TextReplace {

	protected $aPatterns = array();
	protected $aReplacements = array();
	protected $aPatternsWithAttrib = array();
	protected $aLoadedFamilies = array();
	protected $aFamilies;
	
	
	/**
	 * Constructeur de la classe
	 *
	 * @param   aFamilies   (array(string)) tableau des familles de tags à charger. NULL pour toutes.
	 * @return	null
	 * @author	Humpf
	 **/
	public function __construct($aFamilies = NULL) {
		$this->aFamilies = $aFamilies;
	}

	/**
	 * Retourne le tableau des familles de tags qui ont été chargées.
	 * Le tableau est de la forme array("nom famille 1" => true, "nom famille 2" => false)
	 * ce qui indique que "nom famille 1" a été chargée mais pas "nom famille 2".
	 *
	 * @return	(array(bool)) tableau des familles de tags chargées. NULL pour toutes.
	 * @author	Humpf
	 **/
	public function getLoadedFamilies() {
		return $this->aLoadedFamilies;
	}

	/**
	 * Charge un fichier de définition des patterns de recherche (tags).
	 *
	 * @param   filename    (string) nom du fichier XML à parser
	 * @return	null
	 * @author	Humpf
	 **/
	public function loadPatterns($filename) {
	$filename = realpath($filename);
		if(!is_file($filename)) return;
		# Parse le fichier XML
		$data = implode('',file($filename));
		$parser = xml_parser_create(PLX_CHARSET);
		xml_parser_set_option($parser,XML_OPTION_CASE_FOLDING, 0);
		xml_parser_set_option($parser,XML_OPTION_SKIP_WHITE, 1);
		if (!xml_parse_into_struct($parser, $data, $values, $iTags)) {
			$error = xml_get_error_code($parser);
			$errorStr = xml_error_string($error);
			$line = xml_get_current_line_number($parser);
			$col = xml_get_current_column_number($parser);
			trigger_error("XML Parsing error: $errorStr, line: $line, col: $col, file: $filename", E_USER_WARNING);
		}
		xml_parser_free($parser);
		unset($data);

		# Si des éléments <family> sont présents
		if (isset($iTags['family'])) {
			for ($i=0; $i < count($iTags['family']); $i++) {
				if ($values[$iTags['family'][$i]]['type'] == 'open'
					AND $values[$iTags['family'][$i]]['level'] == 2
				) {
					$this->parseFamily($iTags['family'][$i], $iTags['family'][$i+1], $values, $iTags);
					$i++;
				}
			}
		}
	}

	/**
	 * Charge une famille de tags.
	 *
	 * @param   indexStart	(int) index de la balise <family> dans $values
	 * @param   indexEnd		(int) index de la balise </family> dans $values
	 * @param   values			(int) tableau des éléments XML
	 * @return	null
	 * @author	Humpf
	 **/
	private function parseFamily($indexStart, $indexEnd, &$values) {
		$familyName = $values[$indexStart]['attributes']['name'];
		if (is_array($this->aFamilies) 
			AND  !in_array($familyName, $this->aFamilies)) {
			# Ne pas charger la famille $familyName
			$this->aLoadedFamilies[$familyName] = false;
		} else {
			$this->aLoadedFamilies[$familyName] = true;
			for ($iTag=$indexStart+1; $iTag < $indexEnd; $iTag++) {
				if ($values[$iTag]['tag'] == 'tag'
					AND $values[$iTag]['type'] != 'close'
					AND $values[$iTag]['level'] == 3
				) {
					$this->parseTag($iTag, $values);
				}
			}
		}
	}

	/**
	 * Charge un tag.
	 *
	 * @param   iTag				(int) index de la balise <tag> dans $values
	 * @param   values			(int) tableau des éléments XML
	 * @return	null
	 * @author	Humpf
	 **/
	private function parseTag(&$iTag, &$values) {
		$attributes = $values[$iTag]['attributes'];
		$tagName = $attributes['name'];
		$tagPattern = $attributes['pattern'];
		@$tagReplace = $attributes['replace'];
		@$tagAttributes = $attributes['attributes'];
		# S'il y a un sous-élément <replace>, il écrase la valeur de l'attribut 'replace' de <tag>
		if ($values[$iTag+1]['tag'] == 'replace'
			AND $values[$iTag+1]['type'] == 'complete'
			AND $values[$iTag+1]['level'] == 4
		) {
			$tagReplace = $values[$iTag+1]['value'];
			$iTag++;
		}
		# S'il n'y a toujours pas de texte de remplacement
		if (!isset($tagReplace)) $tagReplace = '';
		# S'il y a un attribut 'attributes' dans <tag>
		if (isset($tagAttributes)) {
			# Note: il est possible de mettre une expression régulière dans "attributes"
			# à condition que celle-ci définisse les valeurs de "key" et "value"
			switch($tagAttributes) {
				case 'html':
					$tagAttributes = "/[ \t]*(?P<key>[a-z0-9_-]+)[ \t]*=[ \t]*(?P<q>['\"])(?P<value>.*?)(?P=q)/i";
					break;
				case 'bbcode':
					$tagAttributes = "/[ \t]*(?P<key>[a-zA-Z0-9_-]+)[ \t]*=(?P<value>[^|]*)/";
					break;
			}
			$this->aPatternsWithAttrib[$tagName] = array(
				'pattern' => $tagPattern,
				'replace' => $tagReplace,
				'attributes' => $tagAttributes
			);
			# Supprime les éventuels doublons $tagName
			unset($this->aPatterns[$tagName]);
			unset($this->aReplacements[$tagName]);
		} else {
			$this->aPatterns[$tagName] = $tagPattern;
			$this->aReplacements[$tagName] = $tagReplace;
			# Supprime les éventuels doublons $tagName
			unset($this->aPatternsWithAttrib[$tagName]);
		}

	}

	/**
	 * Recherche et remplace les patterns dans le texte source
	 *
	 * @param		$string		(string) Texte source
	 * @return	string		Texte modifié
	 * @author	Humpf
	 **/
	public function replace($string) {
		# Substitue les patterns dont les chaînes de remplacement sont sans attributs
		$output = preg_replace($this->aPatterns, $this->aReplacements, $string);
		
		# Substitue les patterns dont les chaînes de remplacement sont avec attributs
		foreach ($this->aPatternsWithAttrib as $tag) {
			$cb = new TextReplaceCallback($tag);
			$output = preg_replace_callback($tag['pattern'], array(&$cb, 'replace'), $output);
		}
		return $output;
	}
	
} // class TextReplace

/**
 * Classe de callback pour remplacer les tags munis d'attributs
 *
 * @version 1.0
 * @date	08/01/2012
 * @author	Humpf
 */
class TextReplaceCallback {

	private $tag;
	private $attributes;
	
	/**
	 * Constructeur de la classe
	 *
	 * @param   tag   (array(string)) tableau de description du tag en cours.
	 * @return	null
	 * @author	Humpf
	 **/
	public function __construct($tag) {
		$this->tag = $tag;
	}
	
	/**
	 * Fonction principale qui substitue "%{data}" par la valeur de "data"
	 * dans le texte de remplacement.
	 *
	 * @param 	$matches	(array) mappings du pattern principal. Le premier élément doit être une chaîne qui contient tous les attributs
	 * @return	string		texte de remplacement sans "%{...}"
	 * @author	Humpf
	 */
	public function replace($matches) {
		if (!isset($matches[1])) return '';
		# Extraction des attributs
		preg_match_all($this->tag['attributes'], $matches[1], $submatches, PREG_SET_ORDER);
		$this->attributes = array();
		foreach($submatches as $submatch) {
			$this->attributes[$submatch['key']] = $submatch['value'];
		}
		# Remplacement des attributs %{...}
		$result = preg_replace_callback('/%{([^}]+)}/', array(&$this, 'replaceAttribute'), $this->tag['replace']);
		return $result;
	}
	
	/**
	 * Second niveau de callback pour déterminer la valeur d'un attribut
	 *
	 * @param 	$matches	(array) mappings du pattern secondaire
	 * @return	string		Valeur de l'attribut ou '' si inconnu
	 * @author	Humpf
	 */
	private function replaceAttribute($matches) {
		return isset($this->attributes[$matches[1]]) ? $this->attributes[$matches[1]] : '';
	}
	
} // class TextReplaceCallback

/*
 * Copyright (c) 2012, Humpf
 *
 * This file is part of plxTextReplace.
 * plxTextReplace is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * plxTextReplace is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with plxTextReplace.  If not, see <http://www.gnu.org/licenses/>.
 */
?>